<?php

namespace B2buy\MasterSupplier\Providers;

use Illuminate\Support\ServiceProvider;

class MasterSupplierServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'master_supplier');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'master_supplier');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerMasterSupplier();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerMasterSupplier()
    {
        $this->app->bind('master_supplier', function($app) {
            return new MasterSupplier($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('MasterSupplier', 'Lavalite\MasterSupplier\Facades\MasterSupplier');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('b2buy.master_supplier', function ($app) {
            return $this->app->make('B2buy\MasterSupplier\MasterSupplier');
        });

                // Bind MasterSupplier to repository
        $this->app->bind(
            'B2buy\MasterSupplier\Interfaces\MasterSupplierRepositoryInterface',
            \B2buy\MasterSupplier\Repositories\Eloquent\MasterSupplierRepository::class
        );

        $this->app->register(\B2buy\MasterSupplier\Providers\AuthServiceProvider::class);
                $this->app->register(\B2buy\MasterSupplier\Providers\EventServiceProvider::class);
        
        $this->app->register(\B2buy\MasterSupplier\Providers\RouteServiceProvider::class);
                // $this->app->register(\B2buy\MasterSupplier\Providers\WorkflowServiceProvider::class);
            }

    /**
     * Merges user's and master_supplier's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'b2buy.master_supplier'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeMasterSupplier::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['b2buy.master_supplier'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('b2buy/master_supplier.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/master_supplier')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/master_supplier')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
